using System;
using System.Drawing;
using System.Drawing.Drawing2D;

using UMD.HCIL.Piccolo;
using UMD.HCIL.Piccolo.Util;

namespace Piccolo_1D_Puzzle {
	/// <summary>
	/// A custom Piccolo node that renders a labeled or blank square tile.
	/// It knows how to move itself (with animation) to its appropriate visual
	/// location based on its position within the game.
	/// 
	/// Ben Bederson
	/// April 2005
	/// University of Maryland
	/// </summary>
	public class PTile : PNode {
		static int ANIMATION_TIME = 750;

		static Font TILE_FONT = new Font("Arial", 20, FontStyle.Bold);
		static int TILE_SIZE = 50;
		static int TILE_SPACE = 10;
		static Brush TILE_BRUSH = Brushes.LightBlue;
		static Brush BLANK_TILE_BRUSH = Brushes.LightGray;

		private int position;			// The zero-based index of the tile in the game
		private string label;			// The label displayed on the tile
		private bool blank = false;		// True if the tile is displayed as "blank"

		public PTile(string label) {
			Label = label;
			Brush = TILE_BRUSH;
		}

		public string Label {
			get { return label; }
			set {
				label = value;
				InvalidatePaint();
			}
		}

		public int TilePosition {
			get { return position; }
			set {
				position = value; 
				AnimateToBounds(position * (TILE_SIZE + TILE_SPACE), 0, TILE_SIZE, TILE_SIZE, ANIMATION_TIME);
			}
		}

		public bool Blank {
			get { return blank; }
			set {
				blank = value;
				if (blank) {
					Brush = BLANK_TILE_BRUSH;
				} else {
					Brush = TILE_BRUSH;
				}
				InvalidatePaint();
			}
		}

		protected override void Paint(PPaintContext paintContext) {
			base.Paint(paintContext);		// Let the base method take care of filling the rectangle

			if (!blank) {
				// Draw the label (as long as the tile isn't blank
				SizeF stringSize = paintContext.Graphics.MeasureString(label, TILE_FONT);
				float x = Bounds.X + (Bounds.Width / 2) - (stringSize.Width / 2);
				float y = Bounds.Y + (Bounds.Height / 2) - (stringSize.Height / 2);
				paintContext.Graphics.DrawString(label, TILE_FONT, Brushes.Black, x, y);

				// Debugging code that paints the position of this node in the parent's list of children
				int i = 0;
				foreach (PNode node in Parent) {
					if (node == this) {
						paintContext.Graphics.DrawString(i.ToString(), new Font("Arial", 10), Brushes.Gray, Bounds.X, Bounds.Y);
						break;
					}
					i++;
				}
			}
		}
	}
}